import csv
import struct
import os

datafolder = "B:/theendisnigh/prototype/data/"
mapfolder = "B:/theendisnigh/prototype/tilemaps/"

levels = dict()
worlds = dict()

map = dict()

#tile values
LIFE = 237
CART = 238
BIGLIFE = 311
KEY = 273
LOCK = 33
COIN = 300


EXIT_R1 = 102
EXIT_L1 = 103
EXIT_U1 = 104
EXIT_D1 = 105

EXIT_R2 = 154
EXIT_L2 = 155
EXIT_U2 = 156
EXIT_D2 = 157

EXIT_RU1 = 199
EXIT_RD1 = 200
EXIT_LU1 = 201
EXIT_LD1 = 202

EXIT_RU2 = 195
EXIT_RD2 = 196
EXIT_LU2 = 197
EXIT_LD2 = 198

class levelstat(object):
    life_ct = 0
    cart_ct = 0
    biglife_ct = 0
    key_ct = 0
    lock_ct = 0
    coin_ct = 0
    
    location = (0,0)
    
    
    exit_r = 0
    exit_l = 0
    exit_u = 0
    exit_d = 0
    exit_ru = 0
    exit_rd = 0
    exit_lu = 0
    exit_ld = 0
    
    def to_str(self):
        return ("life_ct: "+str(self.life_ct)+", "+
                "cart_ct: "+str(self.cart_ct)+", "+
                "biglife_ct: "+str(self.biglife_ct)+", "+
                "key_ct: "+str(self.key_ct)+", "+
                "lock_ct: "+str(self.lock_ct)+", "+
                "coin_ct: "+str(self.coin_ct))
                
                
    def get_connection (self, direction):
        for i in range(1,5):
            checker = (self.location[0]+direction[0]*i, self.location[1]+direction[1]*i)
            if checker in map:
                if(map[checker] == ""):
                    return "none"
                    
                if map[checker] != "..":
                    return map[checker]
            else :
                return "none"
        return "none"
        

total_stats = levelstat()
    
def readint(file):
    return struct.unpack('>i', file.read(4))[0]
    
    
def TranslateTile(tileid):
        if(tileid >= 50000):
            return tileid-50000+349 #Tile::BG_TILES_BEGIN;
        if(tileid >= 40000):
            return tileid-40000+200 #Tile::ENEMIES_BEGIN;
        if(tileid >= 30000):
            return tileid-30000+150 #Tile::OVERLAY_TILES_BEGIN;
        if(tileid >= 20000):
            return tileid-20000+100 #Tile::TAG_TILES_BEGIN;
        if(tileid >= 10000):
            return tileid-10000+50 #Tile::TOGGLE_TILES_BEGIN;
        if(tileid >= 0):
            return tileid-0+0 #Tile::BASIC_TILES_BEGIN;
        return tileid;


def parse_level(filename, x, y):
    loadpath = filename.replace("$", "")

    if(os.path.isfile(mapfolder+loadpath)):
        file = open(mapfolder+loadpath, "rb")
        version = readint(file) #version
        width = readint(file)
        height = readint(file)
        nlayers = readint(file)
        
        file.read(width*height*4) #background layer, skip straight to FG
        
        levels[filename] = levelstat()
        
        levels[filename].location = (x, y)
        
        for i in range(0, width*height):
            tileval = TranslateTile(readint(file))
            if(tileval == LIFE):
                levels[filename].life_ct += 1
            if(tileval == CART):
                levels[filename].cart_ct += 1
            if(tileval == BIGLIFE):
                levels[filename].biglife_ct += 1
            if(tileval == KEY):
                levels[filename].key_ct += 1
            if(tileval == LOCK):
                levels[filename].lock_ct += 1
            if(tileval == COIN):
                levels[filename].coin_ct += 1
                
        for i in range(0, width*height):  
            tileval = TranslateTile(readint(file))        
            if(tileval == EXIT_L1 or tileval == EXIT_L2):
                levels[filename].exit_l += 1
            if(tileval == EXIT_R1 or tileval == EXIT_R2):
                levels[filename].exit_r += 1
            if(tileval == EXIT_U1 or tileval == EXIT_U2):
                levels[filename].exit_u += 1
            if(tileval == EXIT_D1 or tileval == EXIT_D2):
                levels[filename].exit_d += 1
        
            if(tileval == EXIT_RU1 or tileval == EXIT_RU2):
                levels[filename].exit_ru += 1
            if(tileval == EXIT_RD1 or tileval == EXIT_RD2):
                levels[filename].exit_rd += 1
            if(tileval == EXIT_LU1 or tileval == EXIT_LU2):
                levels[filename].exit_lu += 1
            if(tileval == EXIT_LD1 or tileval == EXIT_LD2):
                levels[filename].exit_ld += 1
    else: 
        print("No level named "+filename)
      
with open(datafolder+"map.csv") as csvfile:
    reader = csv.reader(csvfile)
    x = 0
    y = 0
    for row in reader:
        x = 0
        y += 1
        for cell in row :
            x += 1
            
            map[(x, y)] = cell
            if(len(cell) > 3):
                parse_level(cell, x, y)
                
outfile = open("wrong_exits.txt", "w")   
levels["none"] = levelstat()
        
for level in sorted(levels):
    
    if(levels[level].exit_r > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((1, 0))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_r != levels[nextlev].exit_l):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (Right):     "+level + " to " + nextlev + " [ "+str(levels[level].exit_r)+", "+str(levels[nextlev].exit_l)+"]", file=outfile)
            
            
    if(levels[level].exit_l > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((-1, 0))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_l != levels[nextlev].exit_r):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (Left):      "+level + " to " + nextlev+ " [ "+str(levels[level].exit_l)+", "+str(levels[nextlev].exit_r)+"]", file=outfile)
    
    
    if(levels[level].exit_u > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((0, -1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_u != levels[nextlev].exit_d):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (Up):        "+level + " to " + nextlev+ " [ "+str(levels[level].exit_u)+", "+str(levels[nextlev].exit_d)+"]", file=outfile)

    if(levels[level].exit_d > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((0, 1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_d != levels[nextlev].exit_u):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (Down):      "+level + " to " + nextlev+ " [ "+str(levels[level].exit_d)+", "+str(levels[nextlev].exit_u)+"]", file=outfile)
            
            
            
    if(levels[level].exit_ru > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((1, -1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_ru != levels[nextlev].exit_ld):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (RightUp):   "+level + " to " + nextlev+ " [ "+str(levels[level].exit_ru)+", "+str(levels[nextlev].exit_ld)+"]", file=outfile)
            
    if(levels[level].exit_rd > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((1, 1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_rd != levels[nextlev].exit_lu):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (RightDown): "+level + " to " + nextlev+ " [ "+str(levels[level].exit_rd)+", "+str(levels[nextlev].exit_lu)+"]", file=outfile)
            
            
    if(levels[level].exit_lu > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((-1, -1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_lu != levels[nextlev].exit_rd):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (LeftUp):    "+level + " to " + nextlev+ " [ "+str(levels[level].exit_lu)+", "+str(levels[nextlev].exit_rd)+"]", file=outfile)
            
    if(levels[level].exit_ld > 0):
        correct_exits = True
        nextlev = levels[level].get_connection((-1, 1))
        if(nextlev == "none"):
            correct_exits = False
        elif(levels[level].exit_ld != levels[nextlev].exit_ru):
            correct_exits = False
            
        if(not correct_exits):
            print("Incorrect Exits (LeftDown):  "+level + " to " + nextlev+ " [ "+str(levels[level].exit_ld)+", "+str(levels[nextlev].exit_ru)+"]", file=outfile)
                
