import csv
import struct
import os

datafolder = "B:/theendisnigh/prototype/data/"
mapfolder = "B:/theendisnigh/prototype/tilemaps/"

levels = dict()
worlds = dict()

#tile values
LIFE = 237
CART = 238
BIGLIFE = 311
KEY = 273
LOCK = 33
COIN = 300

class levelstat(object):
    life_ct = 0
    cart_ct = 0
    biglife_ct = 0
    key_ct = 0
    lock_ct = 0
    coin_ct = 0
    
    def to_str(self):
        return ("life_ct: "+str(self.life_ct)+", "+
                "cart_ct: "+str(self.cart_ct)+", "+
                "biglife_ct: "+str(self.biglife_ct)+", "+
                "key_ct: "+str(self.key_ct)+", "+
                "lock_ct: "+str(self.lock_ct)+", "+
                "coin_ct: "+str(self.coin_ct))

total_stats = levelstat()
    
def readint(file):
    return struct.unpack('>i', file.read(4))[0]
    
    
def TranslateTile(tileid):
        if(tileid >= 50000):
            return tileid-50000+349 #Tile::BG_TILES_BEGIN;
        if(tileid >= 40000):
            return tileid-40000+200 #Tile::ENEMIES_BEGIN;
        if(tileid >= 30000):
            return tileid-30000+150 #Tile::OVERLAY_TILES_BEGIN;
        if(tileid >= 20000):
            return tileid-20000+100 #Tile::TAG_TILES_BEGIN;
        if(tileid >= 10000):
            return tileid-10000+50 #Tile::TOGGLE_TILES_BEGIN;
        if(tileid >= 0):
            return tileid-0+0 #Tile::BASIC_TILES_BEGIN;
        return tileid;


def parse_level(filename):
    loadpath = filename.replace("$", "")
    
    if(os.path.isfile(mapfolder+loadpath)):
        file = open(mapfolder+loadpath, "rb")
        version = readint(file) #version
        width = readint(file)
        height = readint(file)
        nlayers = readint(file)
        
        file.read(width*height*4) #background layer, skip straight to FG
        
        levels[filename] = levelstat()
        
        for i in range(0, width*height):
            tileval = TranslateTile(readint(file))
            if(tileval == LIFE):
                levels[filename].life_ct += 1
            if(tileval == CART):
                levels[filename].cart_ct += 1
            if(tileval == BIGLIFE):
                levels[filename].biglife_ct += 1
            if(tileval == KEY):
                levels[filename].key_ct += 1
            if(tileval == LOCK):
                levels[filename].lock_ct += 1
            if(tileval == COIN):
                levels[filename].coin_ct += 1
        
    #else: 
        #print("No level named "+filename)


        
        
with open(datafolder+"map.csv") as csvfile:
    reader = csv.reader(csvfile)
    for row in reader:
        for cell in row :
            if(len(cell) > 3):
                parse_level(cell)
                
namesout = open("levelinfo.txt", "w")
statout = open("stats.txt", "w")
biglifelevels = open("biglives.txt", "w")

cartlevels = open("cartlevels.txt", "w")
                
for level in sorted(levels):
    worldname = level.split("-")[0]
    lname = level.split(".")[0]
    
    if not worldname in worlds:
        worlds[worldname] = levelstat()
        
    worlds[worldname].life_ct += levels[level].life_ct
    worlds[worldname].cart_ct += levels[level].cart_ct
    worlds[worldname].biglife_ct += levels[level].biglife_ct
    worlds[worldname].key_ct += levels[level].key_ct
    worlds[worldname].lock_ct += levels[level].lock_ct
    worlds[worldname].coin_ct += levels[level].coin_ct
    
    total_stats.life_ct += levels[level].life_ct
    total_stats.cart_ct += levels[level].cart_ct
    total_stats.biglife_ct += levels[level].biglife_ct
    total_stats.key_ct += levels[level].key_ct
    total_stats.lock_ct += levels[level].lock_ct
    total_stats.coin_ct += levels[level].coin_ct
    
    lname2 = lname.replace("$", "")
    
    if(lname[0] != "$" or (lname2+".lvl") not in levels):
        id = "-1"
        cstr = ""
        if (levels[level].life_ct > 0):
            id = "0"
            cstr += " #life"
        if (levels[level].cart_ct > 0):
            id = "0"
            cstr += " #cart"
        if (levels[level].biglife_ct > 0):
            id = "0"
            cstr += " #biglife"
        print("\""+lname2+"\" {name=\"???\" id="+id+"}"+cstr, file=namesout)
        
    if(levels[level].biglife_ct > 0):
        print(level, file=biglifelevels)

    if(levels[level].cart_ct > 0):
        print(level, file=cartlevels)

for world in sorted(worlds):
    print("WORLD "+world+":", file=statout)
    print("    "+worlds[world].to_str(), file=statout)
    print("", file=statout)
    
print ("TOTAL:", file=statout)
print("    "+total_stats.to_str(), file=statout)
                
